"""Delaunay triangulation and interpolation tools.

:Author: Robert Kern <robert.kern@gmail.com>
:Copyright: Copyright 2005 Robert Kern.
:License: BSD-style license. See LICENSE.txt in the scipy source directory.
"""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import six

from matplotlib._delaunay import delaunay
from .triangulate import *
from .interpolate import *
__version__ = "0.2"
