.. matplotlib
.. _matplotlib: http://matplotlib.org
.. _`matplotlib github`: http://github.com/matplotlib/matplotlib

.. _`matplotlib mailing list`: https://lists.sourceforge.net/lists/listinfo/matplotlib-devel
