/* -*- mode: c; c-basic-offset: 4 -*- */

/*
 * Modified for use within matplotlib
 * 5 July 2007
 * Michael Droettboom
 */

/*
** ~ppr/src/include/global_defines.h
** Copyright 1995, Trinity College Computing Center.
** Written by David Chappell.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software and documentation are provided "as is" without
** express or implied warranty.
**
** The PPR project was begun 28 December 1992.
**
** There are many things in this file you may want to change.  This file
** should be the first include file.  It is the header file for the whole
** project.
**
** This file was last modified 22 December 1995.
*/

/*
** TRUE and FALSE
**  The code makes liberal use of these macros.
*/
#if !defined(FALSE)
#define FALSE 0
#endif
#if !defined(TRUE)
#define TRUE !FALSE
#endif

/* end of file */
